@@ -30,20 +30,34 @@ this program.  If not, see <http://www.gnu.org/licenses/>.  */
 # include <sys/file.h>
 #endif
 
+#ifdef WINDOWS32
+# include <windows.h>
+# include "sub_proc.h"
+#endif /* WINDOWS32 */
+
 struct output *output_context = NULL;
-static unsigned int stdio_traced = 0;
+unsigned int stdio_traced = 0;
 
 #define OUTPUT_NONE (-1)
 
 #define OUTPUT_ISSET(_out) ((_out)->out >= 0 || (_out)->err >= 0)
 
+#ifdef HAVE_FCNTL
+# define STREAM_OK(_s)    ((fcntl (fileno (_s), F_GETFD) != -1) || (errno != EBADF))
+#else
+# define STREAM_OK(_s)    1
+#endif
+
 /* I really want to move to gnulib.  However, this is a big undertaking
    especially for non-UNIX platforms: how to get bootstrapping to work, etc.
    I don't want to take the time to do it right now.  Use a hack to get a
    useful version of vsnprintf() for Windows.  */
+#ifdef __VMS
+# define va_copy(_d, _s) ((_d) = (_s))
+#endif
 #ifdef _MSC_VER
-#define va_copy(_d, _s) ((_d) = (_s))
-#define snprintf msc_vsnprintf
+# define va_copy(_d, _s) ((_d) = (_s))
+# define snprintf msc_vsnprintf
 static int
 msc_vsnprintf (char *str, size_t size, const char *format, va_list ap)
 {
@@ -173,13 +187,11 @@ set_append_mode (int fd)
 }
 
 
-#ifdef OUTPUT_SYNC
+#ifndef NO_OUTPUT_SYNC
 
 /* Semaphore for use in -j mode with output_sync. */
 static sync_handle_t sync_handle = -1;
 
-#define STREAM_OK(_s)    ((fcntl (fileno (_s), F_GETFD) != -1) || (errno != EBADF))
-
 #define FD_NOT_EMPTY(_f) ((_f) != OUTPUT_NONE && lseek ((_f), 0, SEEK_END) > 0)
 
 /* Set up the sync handle.  Disables output_sync on error.  */
@@ -369,15 +381,14 @@ output_dump (struct output *out)
       int traced = 0;
 
       /* Try to acquire the semaphore.  If it fails, dump the output
-         unsynchronized; still better than silently discarding it.  */
+         unsynchronized; still better than silently discarding it.
+         We want to keep this lock for as little time as possible.  */
       void *sem = acquire_semaphore ();
 
       /* Log the working directory for this dump.  */
       if (print_directory_flag && output_sync != OUTPUT_SYNC_RECURSE)
         traced = log_working_directory (output_context, 1);
 
-      /* We've entered the "critical section" during which a lock is held.  We
-         want to keep it as short as possible.  */
       if (outfd_not_empty)
         pump_from_tmp (out->out, stdout);
       if (errfd_not_empty && out->err != out->out)
@@ -405,7 +416,7 @@ output_dump (struct output *out)
         }
     }
 }
-#endif /* OUTPUT_SYNC */
+#endif /* NO_OUTPUT_SYNC */
 
 
 /* Provide support for temporary files.  */
@@ -551,7 +562,7 @@ output_close (struct output *out)
       return;
     }
 
-#ifdef OUTPUT_SYNC
+#ifndef NO_OUTPUT_SYNC
   output_dump (out);
 #endif
 
@@ -567,7 +578,7 @@ output_close (struct output *out)
 void
 output_start ()
 {
-#ifdef OUTPUT_SYNC
+#ifndef NO_OUTPUT_SYNC
   if (output_context && output_context->syncout && ! OUTPUT_ISSET(output_context))
     setup_tmpfile (output_context);
 #endif
